searchData={"items":[{"type":"module","title":"Arrows","doc":"A handful of (mostly) arrow macros with superpowers.","ref":"Arrows.html"},{"type":"module","title":"Installation - Arrows","doc":"The package can be installed by adding `arrows` to your list of dependencies in `mix.exs`: \n\n```elixir\ndef deps do\n  [\n    {:arrows, \"~> 0.2.0\"}\n  ]\nend\n```\n\nOr via git:\n```elixir\ndef deps do\n  [\n    {:arrows, git: \"https://github.com/bonfire-networks/arrows\", branch: \"main\"}\n  ]\nend\n```","ref":"Arrows.html#module-installation"},{"type":"module","title":"Documentation - Arrows","doc":"The Elixir `|>` [(\"pipe\") operator](https://hexdocs.pm/elixir/Kernel.html#%7C%3E/2) is one of the things that seems to get people excited about elixir. Probably in part because you then don't have to keep coming up with function names. Unfortunately it's kind of limiting. The moment you need to pipe a parameter into a position that isn't the first one, it breaks down and you have to drop out of the pipeline format or write a secondary function to handle it.\n\nNot any more! By simply inserting `...` where you would like the value to be inserted, `Arrows` will override where it is placed. This allows you to keep on piping while accommodating that function with the annoying argument order. `Arrows` was inspired by [an existing library](https://hexdocs.pm/magritte/Magritte.html).","ref":"Arrows.html#module-documentation"},{"type":"module","title":"Examples - Arrows","doc":"# Standard first position pipe\n    iex> 2 |> Integer.to_string()\n    \"2\"\n    \n    # Using ellipsis for explicit placement\n    iex> 2 |> Integer.to_string(...)\n    \"2\"\n    \n    # Using ellipsis to place the piped value in a non-first position\n    iex> 3 |> String.pad_leading(\"2\", ..., \"0\")\n    \"002\"\n    \n    # Using the ellipsis multiple times\n    iex> 2 |> Kernel.==(..., ...)\n    true\n    \n    # Nested expressions with ellipsis\n    iex> 2 |> String.pad_leading(Integer.to_string(...), 3, \"0\")\n    \"002\"\n\n\nA few little extra features you might notice here:\n* You can move the parameter into a subexpression, as in `2 |> double_fst(double(...), 1)` where\n  double will be called before the parameter is passed to `double_fst`.\n* You can use `...` multiple times, substituting it in multiple places.\n* The right hand side need not even be a function call, you can use any expression with `...`.\n\n#","ref":"Arrows.html#module-examples"},{"type":"module","title":"Ok-pipe - Arrows","doc":"`Arrows` also provides an `ok-pipe` operator, `~>`, which only pipes into the next function if the result from the last one was considered a success. It's inspired by [OK](https://hexdocs.pm/ok/readme.html), but we have chosen to do things slightly differently so it better fits with our regular pipe.\n\ninput                    | result          |\n:----------------------- | :-------------- |\n`{:ok, x}`               | `fun.(x)`       |\n`{:error, e}`            | `{:error, e}`   |\n`nil`                    | `nil`           |\n`x when not is_nil(x)`   | `fun.(x)`       |\n\nIn the case of a function returning an ok/error tuple being on the left hand side, this is straightforward to determine. In the event of `{:ok, x}`, x will be passed into the right hand side to call. In the event of `{:error, x}`, the result will be `{:error, x}`.\n\nWe also deal with a lot of functions that indicate failure by returning nil. `~>` tries to 'do what I mean' for both of these so you can have one pipe operator to rule them all. If `nil` is a valid result, you must thus be sure to wrap it in an `ok` tuple when it occurs on the left hand side of `~>`.\n\n`|>` and `~>` compose in the way you'd expect; i.e. a `~>` receiving an error tuple or nil will stop executing the rest of the chain of (mixed) pipes.\n\n\nDocumentation can be found at [https://hexdocs.pm/arrows](https://hexdocs.pm/arrows).","ref":"Arrows.html#module-ok-pipe"},{"type":"macro","title":"Arrows.<~>/2","doc":"Error-coalescing operator.\n\nSimilar to the nil-coalescing operator (`|||`), but applies a similar logic of the OK-pipe (`~>`).\n\nIt return the right side value if the left side is:\n- `nil`\n- `:error`\n- `{:error, _}`","ref":"Arrows.html#%3C~%3E/2"},{"type":"macro","title":"Examples - Arrows.<~>/2","doc":"iex> nil   \"default\"\n    \"default\"\n    \n    iex> :error   \"default\"\n    \"default\"\n    \n    iex> {:error, :reason}   \"default\"\n    \"default\"\n    \n    iex> {:ok, \"value\"}   \"default\"\n    {:ok, \"value\"}\n    \n    iex> \"value\"   \"default\"\n    \"value\"\n    \n    iex> false   \"default\"\n    false","ref":"Arrows.html#%3C~%3E/2-examples"},{"type":"function","title":"Arrows.from_ok/1","doc":"Extracts values from OK tuples.\n\n- `{:ok, value}` returns `value`\n- `{:error, _}` returns `nil`\n- `:error` returns `nil`\n- Any other value is returned unchanged","ref":"Arrows.html#from_ok/1"},{"type":"function","title":"Examples - Arrows.from_ok/1","doc":"iex> from_ok({:ok, 123})\n    123\n    \n    iex> from_ok({:error, :reason})\n    nil\n    \n    iex> from_ok(:error)\n    nil\n    \n    iex> from_ok(123)\n    123","ref":"Arrows.html#from_ok/1-examples"},{"type":"function","title":"Arrows.ok/1","doc":"Wraps a value in an OK tuple if it's not already in a result tuple format.\n\n- `{:ok, value}`, `{:error, reason}` and `:error` are returned unchanged\n- Any other value `x` is converted to `{:ok, x}`","ref":"Arrows.html#ok/1"},{"type":"function","title":"Examples - Arrows.ok/1","doc":"iex> ok({:ok, 123})\n    {:ok, 123}\n    \n    iex> ok({:error, :reason})\n    {:error, :reason}\n    \n    iex> ok(:error)\n    :error\n    \n    iex> ok(123)\n    {:ok, 123}","ref":"Arrows.html#ok/1-examples"},{"type":"function","title":"Arrows.ok_or/2","doc":"Wraps a value in an OK tuple or returns an error tuple with a default error.\n\n- `{:ok, value}`, `{:error, reason}` and `:error` are returned unchanged\n- `nil` returns `{:error, err}` where `err` is the default error provided in the second argument\n- Any other value `x` returns `{:ok, x}`","ref":"Arrows.html#ok_or/2"},{"type":"function","title":"Examples - Arrows.ok_or/2","doc":"iex> ok_or({:ok, 123}, :default_error)\n    {:ok, 123}\n    \n    iex> ok_or({:error, :reason}, :default_error)\n    {:error, :reason}\n    \n    iex> ok_or(:error, :default_error)\n    :error\n    \n    iex> ok_or(nil, :default_error)\n    {:error, :default_error}\n    \n    iex> ok_or(123, :default_error)\n    {:ok, 123}","ref":"Arrows.html#ok_or/2-examples"},{"type":"function","title":"Arrows.to_ok/1","doc":"Converts various values to an OK tuple format.\n\n- `{:ok, value}` and `{:error, reason}` are returned unchanged\n- `:error` is returned unchanged\n- `nil` is converted to `:error`\n- Any other value `x` is converted to `{:ok, x}`","ref":"Arrows.html#to_ok/1"},{"type":"function","title":"Examples - Arrows.to_ok/1","doc":"iex> to_ok({:ok, 123})\n    {:ok, 123}\n    \n    iex> to_ok({:error, :reason})\n    {:error, :reason}\n    \n    iex> to_ok(:error)\n    :error\n    \n    iex> to_ok(nil)\n    :error\n    \n    iex> to_ok(123)\n    {:ok, 123}","ref":"Arrows.html#to_ok/1-examples"},{"type":"macro","title":"Arrows.|>/2","doc":"Enhanced pipe operator with support for ellipsis (`...`) placement.\n\nThis is a more flexible drop-in replacement for the standard Elixir pipe operator (`|>`).","ref":"Arrows.html#%7C%3E/2"},{"type":"macro","title":"Special Features - Arrows.|>/2","doc":"* The ellipsis (`...`) will be replaced with the result of evaluating the left-hand side expression.\n* The right-hand side need not be a function; it can be any expression containing the ellipsis (`...`).\n* You may use the ellipsis multiple times, and the left-hand side will be calculated exactly once.\n* If no ellipsis is present, it behaves like the standard pipe operator (placing the value as the first argument).","ref":"Arrows.html#%7C%3E/2-special-features"},{"type":"macro","title":"Examples - Arrows.|>/2","doc":"# Standard first position pipe\n    iex> 2 |> Integer.to_string()\n    \"2\"\n    \n    # Using ellipsis for explicit placement\n    iex> 2 |> Integer.to_string(...)\n    \"2\"\n    \n    # Using ellipsis to place the piped value in a non-first position\n    iex> 3 |> String.pad_leading(\"2\", ..., \"0\")\n    \"002\"\n    \n    # Using the ellipsis multiple times\n    iex> 2 |> Kernel.==(..., ...)\n    true\n    \n    # Nested expressions with ellipsis\n    iex> 2 |> String.pad_leading(Integer.to_string(...), 3, \"0\")\n    \"002\"\n\n    # With expressions and transformations\n    iex> 2 |> (... * 3)\n    6","ref":"Arrows.html#%7C%3E/2-examples"},{"type":"macro","title":"Arrows.|||/2","doc":"Nil-coalescing \"or\" operator.\n\nWorks like the logical OR (`||`), except it only defaults to the right side if the left side is `nil` (whereas `||` also defaults on `false` and other falsy values).","ref":"Arrows.html#%7C%7C%7C/2"},{"type":"macro","title":"Examples - Arrows.|||/2","doc":"iex> nil ||| \"default\"\n    \"default\"\n    \n    iex> false ||| \"default\"\n    false\n    \n    iex> 0 ||| \"default\"\n    0\n    \n    iex> \"\" ||| \"default\"\n    \"\"","ref":"Arrows.html#%7C%7C%7C/2-examples"},{"type":"macro","title":"Arrows.~>/2","doc":"OK-pipe operator.\n\nSimilar to the enhanced pipe (`|>`), but with additional error handling for the following patterns:\n\n- `{:ok, value}` - Extracts `value` and passes it to the right side\n- `{:error, _}` - Passes through unchanged (short-circuits the pipeline)\n- `:error` - Passes through unchanged (short-circuits the pipeline)\n- `nil` - Passes through unchanged (short-circuits the pipeline)\n- Any other value - Passes the value directly to the right side","ref":"Arrows.html#~%3E/2"},{"type":"macro","title":"Examples - Arrows.~>/2","doc":"iex> 2 ~> Integer.to_string()\n    \"2\"\n    \n    iex> {:ok, 2} ~> Integer.to_string() |> String.pad_leading(3, \"0\")\n    \"002\"\n    \n    iex> {:error, :reason} ~> Integer.to_string()\n    {:error, :reason}\n    \n    # Note that this would pass :error to `String.pad_leading` breaking our pipe: \n    # :error ~> Integer.to_string() |> String.pad_leading(2, \"0\")\n    :error\n    # Instead we want to do:\n    iex> :error ~> Integer.to_string() ~> String.pad_leading(2, \"0\")\n    :error\n    \n    iex> nil ~> Integer.to_string()\n    nil\n    \n    iex> 2 ~> (... * 2)\n    4\n    \n    # With a non-standard position using ellipsis\n    iex> 2 ~> Kernel./(3, ...)\n    1.5","ref":"Arrows.html#~%3E/2-examples"}],"content_type":"text/markdown"}